#install.packages("DecisionCurve")
#install.packages("rms")
#install.packages("BioPET")
library(DecisionCurve)
library(rms)

#load (simulated) data for risk of  cancer
data(dcaData)
head(dcaData)

#FITTING RISK MODELS USING LOGISTIC REGRESSION, ASSESSING CALIBRATION, OPTIMISM-CORRECTED PERFORMANCE

# use logistic regression to estimate risk function that uses Age, Sex, and two Markers
f<-lrm(Cancer~Age + Female + Smokes + Marker1 + Marker2, data = dcaData)
print(f)
# C is apparent AUC
# Dxy is Somner's D=2(AUC-0.5)

#prevalance of cancer in the dataset is about 12%
mean(dcaData$Cancer)

#Assess Mean Calibration
#first, get predicted risks for individuals in the dataset
fitted.risks<-predict(f,dcaData,type="fitted")
mean(fitted.risks)

#Assess Logistic Calibration
#model fit must return the expanded design matrix
f<-lrm(Cancer~Age + Female + Smokes + Marker1 + Marker2, data = dcaData, x=TRUE, y=TRUE)
#set the seed for bootstrapping
set.seed(1796)
validate(f, B=400)
#look at the index.corrected column for optimism-corrected estimates
#logistic interept is 0 apparent but optimism-corrected is -0.0438
#logistic slope is 1 apparent but optimism-corrected is 0.9507 - indicating that high risks are over estimated and low risks are underestimated

#Assess Moderate Calibration
#Calibration curve
#use function with the RMS package
#m tells average number of people in each group; dataset has 500 people, so m=50 for deciles
val.prob(predict(f,dcaData,type="fitted"),dcaData[,6], m=50, statloc=FALSE)
#use modified function from Steyerberg grup that includes: confidence intervals and gives histogram of predicted risks separatley for cases and controls
source("P:\\Talks\\2017\\SISCR\\tutorial\\val.prob.ci.June09.r")
val.prob.ci(predict(f,dcaData,type="fitted"),dcaData[,6], m=50)
#to remove the statistics printed in the graph: set statloc=FALSE


##  INCREMENTAL VALUE
#Let's suppose that Marker 1 is and established biomarker and Marker 2 is a potentially useful new biomarker
set.seed(9876)
#DecisionCurve calculations for predicting cancer using age, sex, and smoking status
#set bootstraps = 50 here to reduce computation time.
baseline.model <- decision_curve(Cancer~Age + Female + Smokes + Marker1, #fitting a logistic model
                                data = dcaData,
                                study.design = "cohort",
                                bootstraps = 50)

#plot the Decision curve for baseline model
plot_decision_curve(baseline.model,  curve.names = "baseline model")
plot_decision_curve(baseline.model,  curve.names = "baseline model", xlim=c(0, .55))

#switch to Net Benefit rather than standardized Net Benefit
plot_decision_curve(baseline.model, standardize=F, curve.names = "baseline model", xlim=c(0, 0.55))

#Examine the potential for the new biomarkers to improve Net Benefit
full.model <- decision_curve(Cancer~Age + Female + Smokes + Marker1 + Marker2, data = dcaData, bootstraps = 50)
#since we want to plot more than one curve, we pass a list of 'DecisionCurve' objects to the plot
plot_decision_curve( list(baseline.model, full.model),  curve.names = c("Baseline model", "Full model"), xlim=c(0, .55))

#we can also get a table summarizing results
summary(full.model)

#Plot ROC alternative for full model that includes biomarkers
plot_roc_components(full.model,  xlim = c(0, 0.4),col = c("black", "red"))

#Clinical Impact Plot for full model that includes biomarkers
plot_clinical_impact(full.model, xlim = c(0, .4), col = c("black", "blue"))


 

## EVALUATE TWO RISK MODELS FOR PROGNOSTIC ENRICHMENT OF CLINICAL TRIAL
library(BioPET)
library(pROC)
library(ggplot2)
library(gridExtra)
enrichment.baseline.model<-enrichment_analysis(formula=Cancer~Age+Smokes+Marker1, data=dcaData, cost.screening=50, cost.keeping=1000, reduction.under.treatment=0.7)
plot_enrichment_summaries(enrichment.baseline.model)
enrichment.full.model<-enrichment_analysis(formula=Cancer~Age+Smokes+Marker1+Marker2, data=dcaData, cost.screening=50, cost.keeping=1000, reduction.under.treatment=0.7)
plot_enrichment_summaries(enrichment.full.model)

